==> untahris-2.0-src.tgz: untahris/ball.cpp <==
// Untahris (Common Playground)
// copyright (C) 2006 longuens@users.sourceforge.net
// Released under GNU General Public License (see the file COPYING)
// --------------------------------------------------------------
// Pad & Ball.

struct OPad : TPlayObject {
  public:
  int x;
  int y;
  int y0;
  int y1;
  int ptype;
  OPad(int _owner, int _x, int _y) : 
    x(_x), y(_y) {
    owner = _owner;
    if(x < 512 * g.sizeX) ptype = 0;
    else ptype = 1;
    time = g.time;
    g.objects.push_back(this);
    }
  OPad(TBuffer &b) {
    b.get(owner); 
    b.get(x); b.get(y); b.get(time);
    }
  void save(TBuffer &b) {
    b.putChar(8);
    b.put(owner); 
    b.put(x); b.put(y); b.put(time);
    }
  void go() {
    if(ow()->key[keyUp] && g.map[DeCtr(y-2560-192)][DeCtr(x)] == cFree) {
      y -= 192;
      }
    if(ow()->key[keyDown] && g.map[DeCtr(y+2560+192)][DeCtr(x)] == cFree) {
      y += 192;
      }
    if(ow()->key[keyLeft]) {
      bool ok = true;
      for(int y0=DeCtr(y-2560); y0 <= DeCtr(y+2560); y0++) {
        if(g.map[y0][DeCtr(x-256-96)] != cFree) ok = false;
        if(g.map[y0][DeCtr(x-256-96-1024)] != cFree) ok = false;
        }
      if(ok) x -= 96;
      }
    if(ow()->key[keyRight]) {
      bool ok = true;
      for(int y0=DeCtr(y-2560); y0 <= DeCtr(y+2560); y0++) {
        if(g.map[y0][DeCtr(x+256+96)] != cFree) ok = false;
        if(g.map[y0][DeCtr(x+256+96+1024)] != cFree) ok = false;
        }
      if(ok) x += 96;
      }
    
    y0 = DeCtr(y);
    y1 = DeCtr(y);
    while(g.map[y0-1][DeCtr(x)] == cFree) y0--;
    while(g.map[y1+1][DeCtr(x)] == cFree) y1++;
    
    for(int o=0; o<size(g.objects); o++) if(g.objects[o])
      if(g.objects[o]->reactToPad(this)) return;
    time += spd()/4;
    return;
    }
  void draw() {
    int x0 = CTG(x - 256);
    int x1 = CTG(x + 256);
    int y0 = CTG(y - 2560);
    int y1 = CTG(y + 2560);
    for(int xx=x0; xx<x1; xx++)
    for(int yy=y0; yy<y1; yy++) {
      int c;
      if(xx == x0 || yy == y0 || xx == x1-1 || yy == y1-1)
        c = ow()->color;
      else if(ptype)
        c = cmix(ow()->color, 0, (xx-x0-1)/(x1-x0-3.0));
      else
        c = cmix(0, ow()->color, (xx-x0-1)/(x1-x0-3.0));
      d.putpix(xx, yy, c);
      }
    }
  void reactToKey(PlayerKey pk, bool pressed);
  bool catchesRope(int rx, int ry) {
    if(rx >= x-512 && rx <= x+512 && ry >= y-2560 && ry <= y+2560) {
      return true;
      }
    return false;
    }
  void pullRope(int& rx, int& ry, int ddx, int ddy) {
    rx = x; ry = y;
    }
  };

struct OBall : TPlayObject {
  public:
  int x;
  int y;
  int dir;
  int speed;
  OBall(int _owner, int _x, int _y, int _dir) : 
    x(_x), y(_y), dir(_dir) {
    owner = _owner;
    time = g.time;
    speed = 7;
    g.objects.push_back(this);
    }
  OBall(TBuffer &b) {
    b.get(owner); 
    b.get(x); b.get(y); b.get(dir); b.get(time);
    b.get(speed);
    }
  void save(TBuffer &b) {
    b.putChar(9);
    b.put(owner); 
    b.put(x); b.put(y); b.put(dir); b.put(time);
    b.put(speed);
    }
  int reflect(int di) {
    int newdir = dir;
    int d = dir-di;
    if(d<0) d = -d;
    d%=360;
    if(d>180) d=360-d;
    if(d < 90) { // towards...
      newdir = di+di+180-dir;
      newdir %= 360;
      newdir += 360;
      newdir %= 360;
      }
    return newdir;
    }
  void mayChangeDir(int ux, int uy, int di) {
    int nx = DeCtr(ux);
    int ny = DeCtr(uy);
    if(destroyAt(nx, ny, owner)) {
      CellType m = bigType(nx, ny);
      if(isBonus(m)) {
        if(m == cBonusAmmo) {
          d.playSoundAt(d.sndRed, nx, ny);
          ow()->ammo += 10;
          ow()->score += scoreGain();
          }
        if(m == cBonusLevel) {
          d.playSoundAt(d.sndBlue, nx, ny);
          ow()->level += 10;
          ow()->score += 10*scoreGain();
          }
        if(m == cBonusScore) {
          d.playSoundAt(d.sndGreen, nx, ny);
          ow()->score += 5*scoreGain();
          }
        placeBigRandomly(m, 0);
        clearBig(nx, ny, cFree, 0);
        }
      else 
        d.playSoundAt(d.sndHit, nx, ny);
      dir = reflect(di);
      }
    }
  void go() {
    x += degx[dir];
    y += degy[dir];
    int sdir = dir;
    for(int m=0; m<90; m++) for(int u=0; u<360; u+=90) if(dir == sdir) {
      int di = m+u;
      mayChangeDir(x+4*degx[di], y+4*degy[di], di);
      }
    for(int o=0; o<size(g.objects); o++) if(g.objects[o])
      if(g.objects[o]->smashedByBall(this)) dir = g.rand() % 360;
    time += speed;
    return;
    }
  void draw() {
    DIc(icBall).drawAtCentered(CTG(x), CTG(y));
    }
  bool reactToPad(OPad *pad) {
    int yd = pad->y - y;
    if(x >= pad->x-768 && x <= pad->x+768 && yd >= -2560 && yd <= 2560) {
      d.playSoundAt(d.sndHit, DeCtr(x), DeCtr(y));
      owner = pad->owner;
      int di = 90 + (45 * yd) / 2560;
      if(pad->ptype == 0) di = -di; 
      speed = spd()/6+1;
      dir = reflect(di);
      }
    if(y >= Ctr(pad->y0) && y <= Ctr(pad->y1) && (pad->ptype ? x>pad->x : x<pad->x)) {
      int p = pad->owner;
      d.playSoundAt(d.sndDeath, DeCtr(x), DeCtr(y));
      pad->killedBy(owner);
      if(owner == p) owner = 0;
      return true;
      }
    return false;
    }

  bool reactToThruster(OThruster *enemy) {
    if(calcDistance(x-enemy->x, y-enemy->y) < 8*128 + 4*128) {
      d.playSoundAt(d.sndDeath, DeCtr(enemy->x), DeCtr(enemy->y));
      enemy->killedBy(owner);
      dir = g.rand() % 360;
      return true;
      }
    return false;
    }
  bool reactToBullet(OBullet *bullet) {
    if(calcDistance(x-bullet->x, y-bullet->y) < 4*128) {
      dir = g.rand() % 360;
      return true;
      }
    return false;
    }
  void destroyBalls() {
    g.deleteObject(this);
    }
  bool catchesRope(int rx, int ry) {
    if(calcDistance(x-rx, y-ry) < 4 * 128) {
      return true;
      }
    return false;
    }
  void pullRope(int& rx, int& ry, int &drx, int &dry, int ddx, int ddy) {
    rx = x; ry = y; drx = degx[dir]; dry = degy[dir];
    // todo: adjust dir
    }
  };

void OPad::reactToKey(PlayerKey pk, bool pressed) {
  if(!pressed) return;
  int bdir = 1-ptype-ptype;
  switch(pk) {
    case keyFire: case keyWater:
      if(ow()->useAmmo(2)) {
        d.playSoundAt(d.sndShoot, DeCtr(x), DeCtr(y));
        new OBullet(owner, x+2048*bdir, y-2048, bdir*bulletSpeed(), 0);
        new OBullet(owner, x+2048*bdir, y+2048, bdir*bulletSpeed(), 0);
        }
      break;
    case keyAir: case keyEarth:
      d.playSoundAt(d.sndPlace, DeCtr(x), DeCtr(y));
      new OBall(owner, x+2048*bdir, y, (ptype ? 270 : 90) + g.rand() % 45 - g.rand() % 45);
      break;
    }
  }

==> untahris-3.1-src.tgz: untahris/ball.cpp <==
// Untahris (Common Playground)
// copyright (C) 2006 longuens@users.sourceforge.net
// Released under GNU General Public License (see the file COPYING)
// --------------------------------------------------------------
// Pad & Ball.

struct OPad : TPlayObject {
  public:
  int32 x;
  int32 y;
  int32 y0;
  int32 y1;
  int32 ptype;
  OPad(int _owner, int _x, int _y) : 
    x(_x), y(_y) {
    owner = _owner;
    if(x < 512 * g.sizeX) ptype = 0;
    else ptype = 1;
    time = g.time;
    g.objects.push_back(this);
    }
  OPad(TBuffer &b) {
    b.get(owner); 
    b.get(x); b.get(y); b.get(time);
    }
  void save(TBuffer &b) {
    b.putChar(8);
    b.put(owner); 
    b.put(x); b.put(y); b.put(time);
    }
  void go() {
    if(ow()->key[keyUp] && g.map[DeCtr(y-2560-192)][DeCtr(x)] == cFree) {
      y -= 192;
      }
    if(ow()->key[keyDown] && g.map[DeCtr(y+2560+192)][DeCtr(x)] == cFree) {
      y += 192;
      }
    if(ow()->key[keyLeft]) {
      bool ok = true;
      for(int y0=DeCtr(y-2560); y0 <= DeCtr(y+2560); y0++) {
        if(g.map[y0][DeCtr(x-256-96)] != cFree) ok = false;
        if(g.map[y0][DeCtr(x-256-96-1024)] != cFree) ok = false;
        }
      if(ok) x -= 96;
      }
    if(ow()->key[keyRight]) {
      bool ok = true;
      for(int y0=DeCtr(y-2560); y0 <= DeCtr(y+2560); y0++) {
        if(g.map[y0][DeCtr(x+256+96)] != cFree) ok = false;
        if(g.map[y0][DeCtr(x+256+96+1024)] != cFree) ok = false;
        }
      if(ok) x += 96;
      }
    
    y0 = DeCtr(y);
    y1 = DeCtr(y);
    while(g.map[y0-1][DeCtr(x)] == cFree) y0--;
    while(g.map[y1+1][DeCtr(x)] == cFree) y1++;
    
    for(int o=0; o<size(g.objects); o++) if(g.objects[o])
      if(g.objects[o]->reactToPad(this)) return;
    time += spd()/4;
    return;
    }
  void draw() {
    int x0 = CTG(x - 256);
    int x1 = CTG(x + 256);
    int y0 = CTG(y - 2560);
    int y1 = CTG(y + 2560);
    int gt = c.graphicsType;
    for(int xx=x0; xx<x1; xx++)
    for(int yy=y0; yy<y1; yy++) {
      int c;
      if(xx == x0 || yy == y0 || xx == x1-1 || yy == y1-1)
        c = ow()->color;
      else if(gt == gtLines)
        continue;
      else if(gt == gtRetro)
        c = cmix(ow()->color, 0, .5);
      else if(ptype)
        c = cmix(ow()->color, 0, (xx-x0-1)/(x1-x0-3.0));
      else
        c = cmix(0, ow()->color, (xx-x0-1)/(x1-x0-3.0));
      d.putpix(d.ofsX + xx, d.ofsY + yy, c);
      }
    }
  void reactToKey(PlayerKey pk, bool pressed);
  bool catchesRope(int rx, int ry) {
    if(rx >= x-512 && rx <= x+512 && ry >= y-2560 && ry <= y+2560) {
      return true;
      }
    return false;
    }
  void pullRope(OWorm *worm, int ddx, int ddy);
  };

bool destroyed;

struct OBall : TPlayObject {
  public:
  int32 x;
  int32 y;
  int32 dir;
  int32 speed;
  OBall(int _owner, int _x, int _y, int _dir) : 
    x(_x), y(_y), dir(_dir) {
    owner = _owner;
    time = g.time;
    speed = gc.ballSpeed;
    g.objects.push_back(this);
    }
  OBall(TBuffer &b) {
    b.get(owner); 
    b.get(x); b.get(y); b.get(dir); b.get(time);
    b.get(speed);
    }
  void save(TBuffer &b) {
    b.putChar(9);
    b.put(owner); 
    b.put(x); b.put(y); b.put(dir); b.put(time);
    b.put(speed);
    }
  int reflect(int di) {
    int newdir = dir;
    int d = dir-di;
    if(d<0) d = -d;
    d%=360;
    if(d>180) d=360-d;
    if(d < 90) { // towards...
      newdir = di+di+180-dir;
      newdir %= 360;
      newdir += 360;
      newdir %= 360;
      }
    return newdir;
    }
  void mayChangeDir(int ux, int uy, int di) {
    int nx = DeCtr(ux);
    int ny = DeCtr(uy);
    if(destroyAt(nx, ny, owner, dfBall)) {
      CellType m = bigType(nx, ny);
      if(isBonus(m)) {
        if(gc.ballsTakeBonuses) {
          if(m == cBonusAmmo) {
            d.playSoundAt(d.sndRed, nx, ny);
            ow()->ammo += gc.ec[tnPAD].bonusAmmo;
            ow()->score += scoreGain();
            }
          if(m == cBonusLevel) {
            d.playSoundAt(d.sndBlue, nx, ny);
            ow()->level ++;
            ow()->score += 10*scoreGain();
            }
          if(m == cBonusScore) {
            d.playSoundAt(d.sndGreen, nx, ny);
            ow()->score += 5*scoreGain();
            }
          clearBig(nx, ny, cFree, 0);
          if(gc.regenerate(m))
            placeBigRandomly(m, 0);
          }
        }
      else 
        d.playSoundAt(d.sndHit, nx, ny);
      dir = reflect(di);
      }
    }
  void go() {
    x += degx[dir];
    y += degy[dir];
    int sdir = dir;
    for(int m=0; m<90; m++) for(int u=0; u<360; u+=90) if(dir == sdir) {
      int di = m+u;
      mayChangeDir(x+4*degx[di], y+4*degy[di], di);
      }
    for(int o=0; o<size(g.objects); o++) if(g.objects[o])
      if(g.objects[o]->smashedByBall(this)) dir = g.rand() % 360;
    if(gc.explosionsDestroyBalls) {
      int nx = DeCtr(x), ny = DeCtr(y);
      if(g.map[ny][nx] == cExplosion) {
        g.deleteObject(this);
        return;
        }
      }
    time += speed;
    return;
    }
  void draw() {
    DIc(icBall).drawAtCentered(CTG(x), CTG(y));
    }
  bool reactToPad(OPad *pad) {
    int yd = pad->y - y;
    if(x >= pad->x-768 && x <= pad->x+768 && yd >= -2560 && yd <= 2560) {
      d.playSoundAt(d.sndHit, DeCtr(x), DeCtr(y));
      owner = pad->owner;
      int di = 90 + (45 * yd) / 2560;
      if(pad->ptype == 0) di = -di; 
      speed = spd()/6+1;
      dir = reflect(di);
      }
    if(y >= Ctr(pad->y0) && y <= Ctr(pad->y1) && (pad->ptype ? x>pad->x : x<pad->x)) {
      int p = pad->owner;
      d.playSoundAt(d.sndDeath, DeCtr(x), DeCtr(y));
      pad->killedBy(owner);
      if(owner == p) owner = 0;
      return true;
      }
    return false;
    }

  bool reactToThruster(OThruster *enemy) {
    if(calcDistance(x-enemy->x, y-enemy->y) < 8*128 + 4*128) {
      d.playSoundAt(d.sndDeath, DeCtr(enemy->x), DeCtr(enemy->y));
      enemy->killedBy(owner);
      dir = g.rand() % 360;
      return true;
      }
    return false;
    }
  bool reactToBullet(OBullet *bullet) {
    if(calcDistance(x-bullet->x, y-bullet->y) < 4*128) {
      dir = g.rand() % 360;
      if(gc.bulletsKillBalls) g.deleteObject(this);
      return true;
      }
    return false;
    }
  void reactToCannon(OCannonball *ball) {
    if(calcDistance(x-ball->tx, y-ball->ty) < 4*128 + 512) {
      if(gc.cannonsKillBalls) g.deleteObject(this);
      dir = g.rand() % 360;
      }
    }
  void destroyBalls() {
    g.deleteObject(this);
    }
  bool catchesRope(int rx, int ry) {
    if(calcDistance(x-rx, y-ry) < 4 * 128) {
      return true;
      }
    return false;
    }
  void pullRope(OWorm *worm, int ddx, int ddy);
  };

void OPad::reactToKey(PlayerKey pk, bool pressed) {
  if(!pressed) return;
  int bdir = 1-ptype-ptype;
  switch(pk) {
    case keyFire: case keyWater:
      if(ow()->useAmmo()) {
        d.playSoundAt(d.sndShoot, DeCtr(x), DeCtr(y));
        new OBullet(owner, x+2048*bdir, y-2048, bdir*bulletSpeed(), 0);
        new OBullet(owner, x+2048*bdir, y+2048, bdir*bulletSpeed(), 0);
        }
      break;
    case keyAir: case keyEarth:
      d.playSoundAt(d.sndPlace, DeCtr(x), DeCtr(y));
      new OBall(owner, x+2048*bdir, y, (ptype ? 270 : 90) + g.rand() % 45 - g.rand() % 45);
      break;
    }
  }
